@@ -41,7 +41,7 @@ use strict;
 ##==============================================================================
 ## Globals & Constants
 
-our $VERSION = "0.10.007";
+our $VERSION = "0.10.008";
 our @ISA = qw(DiaColloDB::Client);
 
 ## $PGOOD_DEFAULT
@@ -137,6 +137,8 @@ our %TDF_OPTS = (
 ##    tfmin => $tfmin,    ##-- minimum global term-frequency WITHOUT date component (default=2)
 ##    fmin_${a} => $fmin, ##-- minimum independent frequency for value of attribute ${a} (default=undef:from $tfmin)
 ##    keeptmp => $bool,   ##-- keep temporary files? (default=0)
+##    mmap => $bool,      ##-- use mmap() subclasses if available? (default: true)
+##    debug => $bool,     ##-- enable painful debugging code? (default: false)
 ##    index_tdf => $bool, ##-- tdf: create/use (term x document) frequency matrix index? (default=undef: if available)
 ##    index_cof => $bool, ##-- cof: create/use co-frequency index (default=1)
 ##    index_xf => $bool,  ##-- xf: create/use unigram index (default=1)
@@ -212,6 +214,8 @@ sub new {
 		      cfmin => 2,
 		      tfmin => 2,
 		      #keeptmp => 0,
+		      #mmap => 1,
+		      #debug => 0,
 		      index_tdf => undef,
 		      index_cof => 1,
 		      index_xf => 1,
@@ -364,20 +368,20 @@ sub open {
   foreach my $attr (@$attrs) {
     ##-- open: ${attr}*
     my $abase = (-r "$dbdir/${attr}_enum.hdr" ? "$dbdir/${attr}_" : "$dbdir/${attr}"); ##-- v0.03-compatibility hack
-    $coldb->{"${attr}enum"} = $ECLASS->new(base=>"${abase}enum", %efopts)
+    $coldb->{"${attr}enum"} = $coldb->mmclass($ECLASS)->new(base=>"${abase}enum", %efopts)
       or $coldb->logconfess("open(): failed to open enum ${abase}enum.*: $!");
-    $coldb->{"${attr}2t"} = $MMCLASS->new(base=>"${abase}2t", %mmopts)
+    $coldb->{"${attr}2t"} = $coldb->mmclass($MMCLASS)->new(base=>"${abase}2t", %mmopts)
       or $coldb->logconfess("open(): failed to open expansion multimap ${abase}2x.*: $!");
     $coldb->{"pack_t$attr"} //= "\@${atat}$coldb->{pack_id}";
     $atat += packsize($coldb->{pack_id});
   }
 
   ##-- open: tenum
-  $coldb->{tenum} = $XECLASS->new(base=>"$dbdir/tenum", %efopts, pack_s=>$coldb->{pack_t})
+  $coldb->{tenum} = $coldb->mmclass($XECLASS)->new(base=>"$dbdir/tenum", %efopts, pack_s=>$coldb->{pack_t})
       or $coldb->logconfess("open(): failed to open tuple-enum $dbdir/tenum.*: $!");
 
   ##-- open: xf
-  $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags,
+  $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags, mmap=>$coldb->{mmap},
 						     pack_i=>$coldb->{pack_id}, pack_f=>$coldb->{pack_f}, pack_d=>$coldb->{pack_date}
 						    )
     or $coldb->logconfess("open(): failed to open tuple-unigrams $dbdir/xf.*: $!");
@@ -385,7 +389,7 @@ sub open {
 
   ##-- open: cof
   if ($coldb->{index_cof}//1) {
-    $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags,
+    $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags, mmap=>$coldb->{mmap},
 						       pack_i=>$coldb->{pack_id}, pack_f=>$coldb->{pack_f}, pack_d=>$coldb->{pack_date},
 						       dmax=>$coldb->{dmax}, fmin=>$coldb->{cfmin},
 						      )
@@ -486,7 +490,7 @@ sub create_multimap {
   }
   $_ = pack($pack_mmb, sort {$a<=>$b} unpack($pack_mmb,$_//'')) foreach (@v2ti); ##-- ensure multimap target-sets are sorted
 
-  my $v2t = $MMCLASS->new(base=>$base, flags=>'rw', perms=>$coldb->{perms}, pack_i=>$pack_id, pack_o=>$coldb->{pack_off}, pack_l=>$coldb->{pack_id})
+  my $v2t = $coldb->mmclass($MMCLASS)->new(base=>$base, flags=>'rw', perms=>$coldb->{perms}, pack_i=>$pack_id, pack_o=>$coldb->{pack_off}, pack_l=>$coldb->{pack_id})
     or $coldb->logconfess("create_multimap(): failed to create $base.*: $!");
   $v2t->fromArray(\@v2ti)
     or $coldb->logconfess("create_multimap(): failed to populate $base.*: $!");
@@ -620,6 +624,7 @@ sub create {
   $coldb = $coldb->new() if (!ref($coldb));
   @$coldb{keys %opts} = values %opts;
   my $flags = O_RDWR|O_CREAT|O_TRUNC;
+  my $debug = $coldb->{debug};
 
   ##-- initialize: output directory
   my $dbdir = $coldb->{dbdir}
@@ -664,7 +669,7 @@ sub create {
   my ($attr,$ac);
   foreach (0..$#$attrs) {
     push(@$aconf,$ac={i=>$_, a=>($attr=$attrs->[$_])});
-    $ac->{enum}   = $coldb->{"${attr}enum"} = $ECLASS->new(%efopts);
+    $ac->{enum}   = $coldb->{"${attr}enum"} = $coldb->mmclass($ECLASS)->new(%efopts);
     $ac->{pack_t} = $coldb->{"pack_t$attr"} = '@'.$axpos.$pack_id;
     $ac->{s2i}    = $ac->{enum}{s2i};
     $ac->{ma}     = $1 if ($attr =~ /^(?:meta|doc)\.(.*)$/);
@@ -674,7 +679,7 @@ sub create {
   my @aconfw = grep {!defined($_->{ma})} @$aconf; ##-- token-attributes
 
   ##-- initialize: tuple enum (+dates)
-  my $tenum = $coldb->{tenum} = $XECLASS->new(%efopts, pack_s=>$pack_t);
+  my $tenum = $coldb->{tenum} = $coldb->mmclass($XECLASS)->new(%efopts, pack_s=>$pack_t);
   my $ts2i  = $tenum->{s2i};
   my $nt    = 0;
 
@@ -911,16 +916,18 @@ sub create {
       if ($toki_in == $docoff_in) {
 	##-- update break-indices for tdf
 
-#	##-- BUGHUNT/Birmingham: weird errors around here: Tue, 05 Jul 2016 09:27:11 +0200
-#	$coldb->logconfess("create(): \$doci_cur not defined at \$atokfh line ", $atokfh->input_line_number)
-#	  if (!defined($doci_cur));
-#	$coldb->logconfess("create(): \$toki_out not defined at \$atokfh line ", $atokfh->input_line_number)
-#	  if (!defined($toki_out));
-#	$coldb->logconfess("create(): \$docoff->[\$doci_cur=$doci_cur] not defined at \$atokfh line ", $atokfh->input_line_number)
-#	  if (!defined($docoff->[$doci_cur]));
-#	$coldb->logconfess("create(): next \$docoff_in=\$docoff->[++(\$doci_cur=$doci_cur)] not defined at \$atokfh line ", $atokfh->input_line_number)
-#	  if (!defined($docoff->[$doci_cur+1]));
-#	##--/BUGHUNT
+	if ($debug) {
+	  ##-- BUGHUNT/Birmingham: weird errors around here: Tue, 05 Jul 2016 09:27:11 +0200
+	  $coldb->logconfess("create(): \$doci_cur not defined at \$atokfh line ", $atokfh->input_line_number)
+	    if (!defined($doci_cur));
+	  $coldb->logconfess("create(): \$toki_out not defined at \$atokfh line ", $atokfh->input_line_number)
+	    if (!defined($toki_out));
+	  $coldb->logconfess("create(): \$docoff->[\$doci_cur=$doci_cur] not defined at \$atokfh line ", $atokfh->input_line_number)
+	    if (!defined($docoff->[$doci_cur]));
+	  $coldb->logconfess("create(): next \$docoff_in=\$docoff->[++(\$doci_cur=$doci_cur)] not defined at \$atokfh line ", $atokfh->input_line_number)
+	    if (!defined($docoff->[$doci_cur+1]));
+	  ##--/BUGHUNT
+	}
 
 	$docoff->[$doci_cur] = $toki_out;
 	$docoff_in = $docoff->[++$doci_cur];
@@ -984,7 +991,8 @@ sub create {
   ##-- compute unigrams
   if ($coldb->{index_xf}//1) {
     $coldb->info("creating unigram index $dbdir/xf.*");
-    my $xfdb = $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags, pack_i=>$pack_id, pack_f=>$pack_f, pack_d=>$pack_date)
+    my $xfdb = $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags, mmap=>$coldb->{mmap},
+								  pack_i=>$pack_id, pack_f=>$pack_f, pack_d=>$pack_date)
       or $coldb->logconfess("create(): could not create $dbdir/xf.*: $!");
     $xfdb->create($coldb, $tokfile)
       or $coldb->logconfess("create(): failed to create unigram index: $!");
@@ -995,7 +1003,7 @@ sub create {
   ##-- compute collocation frequencies
   if ($coldb->{index_cof}//1) {
     $coldb->info("creating co-frequency index $dbdir/cof.* [dmax=$coldb->{dmax}, fmin=$coldb->{cfmin}]");
-    my $cof = $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags,
+    my $cof = $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags, mmap=>$coldb->{mmap},
 								 pack_i=>$pack_id, pack_f=>$pack_f, pack_d=>$pack_date,
 								 dmax=>$coldb->{dmax}, fmin=>$coldb->{cfmin},
 								 keeptmp=>$coldb->{keeptmp},
@@ -1125,7 +1133,7 @@ sub union {
   foreach $ac (@$adata) {
     $coldb->vlog($coldb->{logCreate}, "union(): creating attribute enum $dbdir/$ac->{a}_enum.*");
     $attr  = $ac->{a};
-    $aenum = $coldb->{"${attr}enum"} = $ac->{enum} = $ECLASS->new(%efopts);
+    $aenum = $coldb->{"${attr}enum"} = $ac->{enum} = $coldb->mmclass($ECLASS)->new(%efopts);
     $as2i  = $aenum->{s2i};
     foreach $argi (0..$#dbargs) {
       ##-- enum union: guts
@@ -1156,7 +1164,7 @@ sub union {
 
   ##-- union: tenum
   $coldb->vlog($coldb->{logCreate}, "union(): creating tuple-enum $dbdir/tenum.*");
-  my $tenum = $coldb->{tenum} = $XECLASS->new(%efopts, pack_s=>$pack_t);
+  my $tenum = $coldb->{tenum} = $coldb->mmclass($XECLASS)->new(%efopts, pack_s=>$pack_t);
   my $ts2i  = $tenum->{s2i};
   my $nt    = 0;
   foreach $db (@dbargs) {
@@ -1199,7 +1207,7 @@ sub union {
   ##-- unigrams: populate
   if ($coldb->{index_xf}//1) {
     $coldb->vlog($coldb->{logCreate}, "union(): creating tuple unigram index $dbdir/xf.*");
-    $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags,
+    $coldb->{xf} = DiaColloDB::Relation::Unigrams->new(base=>"$dbdir/xf", flags=>$flags, mmap=>$coldb->{mmap},
 						       pack_i=>$pack_id, pack_f=>$pack_f, pack_d=>$pack_date,
 						       keeptmp => $coldb->{keeptmp},
 						      )
@@ -1213,7 +1221,7 @@ sub union {
   ##-- co-frequencies: populate
   if ($coldb->{index_cof}//1) {
     $coldb->vlog($coldb->{logCreate}, "union(): creating co-frequency index $dbdir/cof.* [fmin=$coldb->{cfmin}]");
-    $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags,
+    $coldb->{cof} = DiaColloDB::Relation::Cofreqs->new(base=>"$dbdir/cof", flags=>$flags, mmap=>$coldb->{mmap},
 						       pack_i=>$pack_id, pack_f=>$pack_f, pack_d=>$pack_date,
 						       dmax=>$coldb->{dmax}, fmin=>$coldb->{cfmin},
 						       keeptmp=>$coldb->{keeptmp},
@@ -1280,7 +1288,7 @@ sub union {
 ## @keys = $coldb->headerKeys()
 ##  + keys to save as header
 sub headerKeys {
-  return (qw(attrs upgraded), grep {!ref($_[0]{$_}) && $_ !~ m{^(?:dbdir$|flags$|perms$|info$|tdfopts$|log)}} keys %{$_[0]});
+  return (qw(attrs upgraded), grep {!ref($_[0]{$_}) && $_ !~ m{^(?:dbdir$|flags$|perms$|info$|tdfopts$|log|debug)}} keys %{$_[0]});
 }
 
 ## $bool = $coldb->loadHeaderData()
@@ -1754,7 +1762,7 @@ sub parseQuery {
 		   (?:${reqre})			##-- final component
 		   ${sepre}*			##-- final separators (optional)
 		   $/x) {
-    $coldb->debug("parseQuery($logas): parsing native query request [ddcmode=$ddcmode]");
+    $coldb->debug("parseQuery($logas): parsing native query request [ddcmode=$ddcmode]") if ($coldb->{debug});
     $areqs = [grep {defined($_)} ($req =~ m/${sepre}*(${reqre})/g)];
   }
 
@@ -1783,7 +1791,7 @@ sub parseQuery {
       $attr = $defaultIndex//'' if (($attr//'') eq '');
       $attr =~ s/^\$//;
 
-      $coldb->debug("parseQuery($logas): parsing native request clause: (".($attr//'')." = ".($areq//'').")");
+      $coldb->debug("parseQuery($logas): parsing native request clause: (".($attr//'')." = ".($areq//'').")") if ($coldb->{debug});
 
       if (UNIVERSAL::isa($areq,'DDC::Any::CQuery')) {
 	##-- compat: value: ddc query object
@@ -1856,7 +1864,7 @@ sub parseQuery {
 		       })
     if ($opts{mapand} || (!defined($opts{mapand}) && $req0 !~ /\&\&/));
 
-  $coldb->debug("parseQuery($logas): parsed query: ", $q->toString);
+  $coldb->debug("parseQuery($logas): parsed query: ", $q->toString) if ($coldb->{debug});
 
   return $q;
 }
